package rs.core;

import java.io.File;

import rs.corelibrary.R;
import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Environment;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

public class FileBrowseDialog {
	public static interface OnFileSelectedListener {
		void onFileSelected(File f, boolean isNew);
	};
	
	
	
	private static final String ROOT = "/";
	private static File _folder = Environment.getExternalStorageDirectory();
	private static TextView file;
	private static class FileAdapter extends ArrayAdapter<File> implements OnItemClickListener{
		public FileAdapter(Context ctx) {
			super(ctx,android.R.layout.simple_list_item_1);
			update(_folder);
			
		}

		@Override
		public void onItemClick(AdapterView<?> arg0, View arg1, int position,
				long arg3) {
			File f = getItem(position);
			if(f.getName().equals(".."))
				update(_folder.getParentFile());
			else {
				if(f.isDirectory())
					update(f);
				else
					file.setText(f.getName());
			}
		}
		public void update(File f) {
			 
			_folder = f;
			clear();
			if(!ROOT.equals(f.getName()))
				add(new File(".."));
			if(f.listFiles() != null) {
				for(File file : f.listFiles()) 
					if(file.isDirectory() && file.canWrite()) 
						add(file);
				for(File file : f.listFiles()) 
					if(!file.isDirectory() && file.getName().endsWith(".json"))
						add(file);
			}
			notifyDataSetChanged();
		}
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View v = super.getView(position, convertView, parent);
			((TextView)v).setText(getItem(position).getName());
			return v;
		}
	}
	
	private static FileAdapter ADAPTER;
	@SuppressLint("InflateParams")
	public static void show(Context ctx, final OnFileSelectedListener listener, final boolean createNew) {
		AlertDialog.Builder b = new AlertDialog.Builder(ctx);
		View v = LayoutInflater.from(ctx).inflate(R.layout.file_browser, null);
		file = (TextView)v.findViewById(R.id.ed_filename);
		file.setEnabled(createNew);
		if(createNew)
			file.setText("settings.json");
		b.setView(v);
		b.setPositiveButton(android.R.string.ok, null);
		ListView folders = (ListView)v.findViewById(R.id.lv_folders);
		ADAPTER = new FileAdapter(ctx);
		folders.setAdapter(ADAPTER);
		folders.setOnItemClickListener(ADAPTER);
		final AlertDialog d = b.create();
		d.setOnShowListener(new DialogInterface.OnShowListener() {
			@Override
			public void onShow(DialogInterface dialog) {
				final View ok = d.getButton(DialogInterface.BUTTON_POSITIVE); 
				ok.setOnClickListener(new View.OnClickListener() {
					@Override
					public void onClick(View v) {
						d.dismiss();
						listener.onFileSelected(new File(_folder,file.getText().toString()),createNew);
					}
				});
				file.addTextChangedListener(new TextWatcher() {
					@Override
					public void onTextChanged(CharSequence s, int start, int before, int count) {
						// TODO Auto-generated method stub
						
					}
					
					@Override
					public void beforeTextChanged(CharSequence s, int start, int count,
							int after) {
						// TODO Auto-generated method stub
						
					}
					
					@Override
					public void afterTextChanged(Editable s) {
						ok.setEnabled(!s.toString().isEmpty());
					}
				});
				
			}
		});
		d.show();
	}
}
