package rs.core;

import java.util.EnumMap;
import java.util.Map;

import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;

public class Utils {
	private static final int WHITE = 0xFFFFFFFF;
	private static final int BLACK = 0xFF000000;
	public static final String FIELD_JSON = "JSON";
	public static final String FIELD_KEY = "AKEY";
	
	public static final Intent CORE_SERVICE_INTENT = new Intent("rs.core.service").setPackage("rs.core");
	public static final Uri CONTENT_URI_GET = Uri.parse("content://rs.core.settings/get");
	public static final Uri CONTENT_URI_SET = Uri.parse("content://rs.core.settings/set");
	public static final Uri CONTENT_URI_TEST = Uri.parse("content://rs.core.settings/test");

	public  static Bitmap encodeAsBitmap(String contents, BarcodeFormat format,
			int img_width, int img_height) throws WriterException {
		String contentsToEncode = contents;
		if (contentsToEncode == null) {
			return null;
		}
		Map<EncodeHintType, Object> hints = null;
		hints = new EnumMap<EncodeHintType, Object>(EncodeHintType.class);
		hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");
		MultiFormatWriter writer = new MultiFormatWriter();
		BitMatrix result;
		try {
			result = writer.encode(contentsToEncode, format, img_width,
					img_height, hints);
		} catch (IllegalArgumentException iae) {
			iae.printStackTrace();
			return null;
		}
		int width = result.getWidth();
		int height = result.getHeight();
		int[] pixels = new int[width * height];
		for (int y = 0; y < height; y++) {
			int offset = y * width;
			for (int x = 0; x < width; x++) {
				pixels[offset + x] = result.get(x, y) ? BLACK : WHITE;
			}
		}

		Bitmap bitmap = Bitmap.createBitmap(width, height,
				Bitmap.Config.ARGB_8888);
		bitmap.setPixels(pixels, 0, width, 0, 0, width, height);
		return bitmap;
	}
	public static String dump(byte [] data) {
		if(data == null) return "";
		String result = "";
		for(byte b : data)
			result += String.format("%02X", b);
		return result;
	}

}
