package rs.core.hw;

import android.content.Intent;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.Log;

/**
 * Объект данных - штрихкод
 * @author nick
 *
 */
public class Barcode implements Parcelable,DeviceData {
	/**
	 * Типы баркодов
	 * @author nick
	 *
	 */
	public static enum BarcodeTypes {
		unknown(0),all(-1),ean13(11,55),ean8(10,54),code39(1,67),code93(7),code128(3,111),qr(28),pdf417(17),interleaved2of5(6,50),upca(-117,49,8,48),upce(9);
		
		private final int code[];
		private BarcodeTypes(int...code) {
			this.code = code;
		}
		public static BarcodeTypes decodeType(int value) {
			for(BarcodeTypes t : values())
				for(int i=0;i<t.code.length;i++)
					if(t.code[i] == value)
						return t;
			return unknown;
		}
		public int code() { return this.code[0]; }
	}
	
	/**
	 * Тип штрихкода
	 */
	public BarcodeTypes type;
	/**
	 * Значение
	 */
	public String value;
	public Barcode(Parcel p) {
		type = BarcodeTypes.values()[p.readInt()];
		value = p.readString();
	}
	public Barcode(Intent intent) {
		if(intent.hasExtra("barocode")) {
			this.value = new String(intent.getByteArrayExtra("barocode"),0,intent.getIntExtra("length", 0));
			this.type = BarcodeTypes.decodeType(intent.getByteExtra("barcodeType", (byte)0));
		} else {
			this.value = intent.getExtras().get("se4500").toString();
			this.type = BarcodeTypes.code128;
		}
		Log.d("RSCore","Got "+this.type+"  for "+intent.getByteExtra("barcodeType", (byte)0));
	}
	public Barcode(int type, String value) {
		this.value = value;
		this.type = BarcodeTypes.decodeType(type);
	}
	public Barcode(int type, int length, byte []code) {
		this.type = BarcodeTypes.decodeType(type);
		this.value = new String(code,0,length);
	}
	@Override
	public String toString() {
		return value+" ("+type+")";
	}
	@Override
	public int describeContents() {
		return 0;
	}
	@Override
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeInt(type.ordinal());
		dest.writeString(value);
	}
	
	public static final Parcelable.Creator<Barcode> CREATOR = new Parcelable.Creator<Barcode>() {

		@Override
		public Barcode createFromParcel(Parcel source) {
			return new Barcode(source);
		}

		@Override
		public Barcode[] newArray(int size) {
			return new Barcode[size];
		}
		
	};
}
