package rs.core.hw;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Field;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Bitmap.CompressFormat;
import settings.AppSettings;

public abstract class IPrinter extends Thread {
	public static final int TYPE_NORMAl = 0;
	public static final int TYPE_BOLD =   1;
	public static final int TYPE_ITALIC = 2;
	public static final int TYPE_BOLD_ITALIC = TYPE_ITALIC | TYPE_BOLD;
	public static final int TYPE_UNDERLINE = 4;
	public static final int TYPE_STRIKEOUT = 8;
	public static final String DEFAULT_TYPEFACE = "monospace";
	public static final int DEFAULT_SIZE = 16;
	public static final float MM_SCALE = 6.62f;
	
	protected final Context _context;
	protected BasePrinterConfig Config; 
	protected File SPOOLER;

	protected class PrintJob {
		public final long start;
		public final BasePrinterConfig CONFIG;
		public final Bitmap BITMAP;
		public PrintJob(Bitmap bitmap, BasePrinterConfig config) {
			start = System.currentTimeMillis();
			CONFIG = config;
			BITMAP = bitmap;
		}
		public void store() {
			File f  = new File(SPOOLER,String.format("%x", start));
			try {
				DataOutputStream ds = new DataOutputStream(new FileOutputStream(f));
				try {
					byte [] b = new byte [] {'R','S','J'};
					ds.write(b);
					b = AppSettings.store(CONFIG, _context).toString().getBytes();
					ds.writeInt(b.length);
					ds.write(b);
					BITMAP.compress(CompressFormat.PNG, 100, ds);
				} finally {
					ds.close();
					BITMAP.recycle();
				}
			} catch(IOException ioe) {
				if(f.exists())
					f.delete();
			}
		}
		public PrintJob(File f) throws Exception {
			DataInputStream ds = new DataInputStream(new FileInputStream(f));
			try {
				byte [] b= new byte[3];
				ds.read(b);
				if(b[0] != 'R' || b[1] != 'S' || b[2] != 'J')
					throw new Exception("Not a job file");
				start = ds.readLong();
				int size_t = ds.readInt();
				b = new byte[size_t];
				ds.read(b);
				CONFIG = createConfigFor(IPrinter.this.getClass());
				AppSettings.loadFromJson(CONFIG, new String(b));
				BITMAP = BitmapFactory.decodeStream(ds);
			} finally { 
				ds.close();
			}
		}
	}
	
	public void addJob(Bitmap job, BasePrinterConfig config) { 
		synchronized (SPOOLER) {
				PrintJob pjob = new PrintJob(job,config);
				File [] files = SPOOLER.listFiles();
				if(files == null || files.length  == 0)
					startJob(pjob);
				else 
					pjob.store();
		}
	}
	protected  abstract void startJob(PrintJob job);
	
	public IPrinter(Context context) {
		_context = context;
		SPOOLER  = new File(_context.getFilesDir(),"spooler/");
		if(!SPOOLER.exists())
			SPOOLER.mkdirs();
	}
	public boolean open(BasePrinterConfig config) {
		start();
		return true;
	}
	public void close() {
		interrupt();
		for(File f : SPOOLER.listFiles())
			if(f.isFile())
				f.delete();
	}
	@Override
	public void run() {
		while(!isInterrupted()) {
			File [] files = null;
			synchronized(SPOOLER) {
				files = SPOOLER.listFiles();
			}
			if(files != null && files.length > 0) try {
				startJob(new PrintJob(files[0]));
			} catch(Exception e) {
				
			} finally {
				files[0].delete();
			}
			try { Thread.sleep(100); } catch(InterruptedException ie) { }
		}
	}
	
	@SuppressWarnings("unchecked")
	public static BasePrinterConfig createConfigFor(Class<? extends IPrinter> c) {
		BasePrinterConfig result = null;
		try {
			Field f = c.getDeclaredField("CONFIG_CLASS");
			f.setAccessible(true);
			result = ((Class<? extends BasePrinterConfig>) f.get(null)).newInstance();
		} catch (Exception nsf) {
			result = new BasePrinterConfig();
		}
		return result;

	}
	
}
