package rs.core.example;

import java.io.IOException;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.util.LinkedList;
import java.util.Queue;



import rs.core.CoreInterface;
import rs.core.example.ui.Startup;
import rs.core.hw.Barcode;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.os.Parcelable;
import android.os.RemoteException;
import android.widget.Toast;

public class Main extends Activity implements RSCoreConnection {
	
	
	
	
	private final String MESSAGE_TAG = getClass().getPackage().getName()+"."+hashCode();
	private static final String CORE_SERVICE_ACTION = "rs.core.service";
	private static final String DATA_EXTRA = "Data";
	private static final String SENDER_EXTRA = "Sender";

	private RSCoreListener _listener;
	
	private class RemoterThread extends Thread {
		private ByteBuffer bb = ByteBuffer.allocate(1024);
		private ServerSocket _s;
		private volatile boolean _hasConnection;
		private Queue<String> BARCODES = new LinkedList<String>();
		@Override
		public void run() {
			try {
				_s = new ServerSocket(9005);
				while(!isInterrupted())  {
					Socket s = _s.accept();
					if(s != null) {
						OutputStream os = s.getOutputStream();
						_hasConnection = true;
						while(!isInterrupted()) {
							String barcode = null;
							synchronized (BARCODES) {
								if(!BARCODES.isEmpty())
									barcode = BARCODES.poll();
							}
							if(barcode != null) {
								bb.clear();
								bb.put((byte)0x46);
								byte [] bytes = barcode.getBytes();
								bb.putInt(bytes.length);
								bb.put(bytes);
								try {
									os.write(bb.array(),0,bb.position());
								} catch(IOException ioe) {
									break;
								}
							}
							try { Thread.sleep(20); } catch(InterruptedException ie) { return; }
						}
						BARCODES.clear();
						_hasConnection = false;
					}
				}
			} catch(IOException ioe) {
				_hasConnection = false;
				BARCODES.clear();
			}
		}
		@Override
		public void interrupt() {
			try {
				_s.close();
			} catch(Exception e) { };
			super.interrupt();
			
		}
		public void addBarcode(String s) {
			if(!_hasConnection) return;
			synchronized (BARCODES) {
				BARCODES.add(s);
			}
		}
		
	}
	
	private RemoterThread REMOTER;
	
	private BroadcastReceiver CORE_EVENT_RECEIVER = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			Parcelable data = intent.getParcelableExtra(DATA_EXTRA);
			if(data instanceof Barcode)
				REMOTER.addBarcode(((Barcode)data).value);
			if(_listener != null)
				_listener.onCoreEvent(intent.getStringExtra(SENDER_EXTRA), data);
			
		}
	};
		
	
	private class CoreServiceConnector implements ServiceConnection {
		private CoreInterface _RSCore;
		@Override
		public void onServiceConnected(ComponentName name, IBinder service) {
			_RSCore = CoreInterface.Stub.asInterface(service);
			
			try {
				_RSCore.subscribe(MESSAGE_TAG);
				String sDevices = _RSCore.availableDevices();
				String sExtensions = _RSCore.getExtensions();
				String sModel = _RSCore.getDeviceModel();
				System.out.println(sDevices+" "+sExtensions+" "+sModel);
				getFragmentManager().beginTransaction().replace(R.id.v_content, new Startup()).commit();
			} catch(RemoteException re) {
				Toast.makeText(Main.this, "Ошибка при вызове метода RSCore "+re.getLocalizedMessage(), Toast.LENGTH_LONG).show();
				
			}
		}

		@Override
		public void onServiceDisconnected(ComponentName name) {
			try {
				_RSCore.unsubscribe(MESSAGE_TAG);
			} catch(RemoteException re) { }

			_RSCore = null;
			
		}
		
	}
	@Override
	public void onBackPressed() {
		if(getFragmentManager().getBackStackEntryCount() > 0)
			getFragmentManager().popBackStack();
		else
			super.onBackPressed();
	}

	private CoreServiceConnector CORE_CONNECTOR = new CoreServiceConnector();
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		registerReceiver(CORE_EVENT_RECEIVER, new IntentFilter(MESSAGE_TAG));
		if(! bindService(new Intent(CORE_SERVICE_ACTION), CORE_CONNECTOR, BIND_AUTO_CREATE) )
			Toast.makeText(Main.this, "Ошибка подключения к RSCore", Toast.LENGTH_LONG).show();
	}
	@Override
	protected void onDestroy() {
		unregisterReceiver(CORE_EVENT_RECEIVER);
		unbindService(CORE_CONNECTOR);
		super.onDestroy();
	}
	@Override
	public void setListener(RSCoreListener l) {
		_listener = l;
		
	}
	@Override
	protected void onStart() {
		super.onStart();
		REMOTER = new RemoterThread();
		REMOTER.start();
		
	}
	@Override
	protected void onStop() {
		REMOTER.interrupt();
		super.onStop();
	}
	@Override
	public void removeListener(RSCoreListener l) {
		if(_listener == l)
			_listener = null;
	}
	@Override
	public CoreInterface getRSCore() {
		return CORE_CONNECTOR._RSCore;
	}

}
