package rs.core.example.exchange;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;


public class ExchangeServer {
	
	public static interface DataListener {
		public void onData(byte command, byte [] data, int size);
	}
	 
	
	private static final int DEFAULT_PORT = 9004;
	private static final int DEFAULT_BLOCK_SIZE = 8192;
	public static final byte CMD_PUT		   	= 0x10;
	public static final byte CMD_PUT_FIRST  	 = 0x1;
	public static final byte CMD_PUT_LAST     = 0x2;

	public static final byte CMD_ECHO  		=0x20;
	public static final byte CMD_CLEAR 		 =0x40;
	public static final byte CMD_GET   		= (byte)0x80;

	private static final byte MODE_COMMAND = 0;
	private static final byte MODE_DATA = 1;
	private DataListener _listener;
	private ServerThread SERVER_THREAD;
	public ExchangeServer(DataListener listener) {
		_listener = listener;
	}
	private class ServerThread extends Thread {
		private ServerSocket server;
		private ByteBuffer CMD_BUFFER = ByteBuffer.allocate(5);
		private byte [] DATA = new byte[DEFAULT_BLOCK_SIZE];
		@Override
		public void run() {
			try {
				server = new ServerSocket(DEFAULT_PORT);
				while (!isInterrupted()) {
					Socket s = server.accept();
					if(s != null) {
						s.setSoTimeout(2000);
						InputStream is = s.getInputStream();
						OutputStream os = s.getOutputStream();
						while(true) {
							try { Thread.sleep(50); } catch(InterruptedException ioe) { return; }
							CMD_BUFFER.clear();
							try {
								int read  = is.read(CMD_BUFFER.array());
								if(read == 0) continue;
								if(read < 0) break;
								if(read != CMD_BUFFER.capacity()) break;
								byte cmd = CMD_BUFFER.get();
								int size = CMD_BUFFER.getInt();
								System.out.println(String.format("Comamnd %x, data size %d",cmd,size));
								if(size > 0 && _listener != null) {
									_listener.onData(CMD_PUT_FIRST, null,  0);
									while(size > 0) {
										read = is.read(DATA);
										_listener.onData(CMD_PUT, DATA, read);
										size-=read;
									}
									_listener.onData(CMD_PUT_LAST, null, 0);
								}
								size = 0;
								byte [] data = null;
								if(cmd == CMD_GET) {
									System.out.println("Returning data");
									data = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><Table><Record BarCode=\"2000000001913\" Quantity=\"1\" /><Record BarCode=\"2000000059747\" Quantity=\"2\"/></Table>".getBytes();
									size = data.length;
/*/    <Record BarCode=\"2000000001913\" Quantity=\"1\" /><Record BarCode=\"2000000059747\" Quantity=\"2\"/> 
</Table>" */
								}
								CMD_BUFFER.clear();
								CMD_BUFFER.put(cmd);
								CMD_BUFFER.putInt(size);
								os.write(CMD_BUFFER.array());
								if(data != null)
									os.write(data);
								
							} catch(IOException ioe) {
								break;
							}
							
						}
						try {
							s.close();
						} catch(IOException ioe) { }
					}
				}
			} catch (IOException ioe) {
			}
			finally {
				SERVER_THREAD = null;
			}
		}
		@Override
		public void interrupt() {
			try {
				server.close();
			} catch(Exception e) { }
			super.interrupt();
		}
	};
	public void start() {
		if(SERVER_THREAD != null)
			return;
		SERVER_THREAD = new ServerThread();
		SERVER_THREAD.start();
	}
	public void stop() {
		SERVER_THREAD.interrupt();
	}

}
