package rs.core.hw;

import java.io.IOException;
import java.util.Arrays;

import rs.core.Utils;
import android.nfc.Tag;
import android.nfc.tech.MifareClassic;
import android.nfc.tech.NfcV;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.Log;

public class NfcTag implements Parcelable,DeviceData {
	
	
	/**
	 * Идентификатор метки
	 */
	public final byte[] ID;
	/**
	 * Данные блоков метки
	 */
	public String DATA = "";
	public int SECTORS;
	public int BLOCKS;
	public int BLOCK_SIZE;
	public String TYPE = "Unknown";

	public NfcTag(Parcel p) {
		ID = new byte[p.readInt()];
		p.readByteArray(ID);
		DATA = p.readString();
		TYPE = p.readString();
	}
	public NfcTag(Tag tag, byte [] key) throws IOException {
		ID = tag.getId();
		for (String s : tag.getTechList())
			if (MifareClassic.class.getName().equals(s)) {
				readMifareClassic(tag,key);
				break;
			}
			else if(NfcV.class.getName().equals(s)) {
				readNfcV(tag);
				break;
			}
	}

	public void readNfcV(Tag tag) throws IOException {
		NfcV nfcvTag = NfcV.get(tag);
		try {        
			
            nfcvTag.connect();
            byte [] cmd = new byte[] { 
            		(byte)0x20,  // FLAGS
            	    (byte)0x20,  // READ_SINGLE_BLOCK
            	    0, 0, 0, 0, 0, 0, 0, 0,
            	    0};
            BLOCKS = 1;
            BLOCK_SIZE = -1;
            SECTORS = 28;
            DATA = "";
            System.arraycopy(tag.getId(), 0, cmd, 2, 8);
            for(int i=0;i<SECTORS;i++) {
            	cmd[10] = (byte)i;
            	byte[] userdata = nfcvTag.transceive(cmd);
            	if(userdata[0] != 0) {
            		SECTORS = i;
            		break;
            	}
            	if(!DATA.isEmpty())
            		DATA+=";";
            	if(BLOCK_SIZE == -1)
            		BLOCK_SIZE = userdata.length-1;
            	String s = Utils.dump(Arrays.copyOfRange(userdata, 1, BLOCK_SIZE+1)); 
            	DATA += s;
            }
            TYPE = "NfcV";
            
		} catch(IOException ioe) {
			Log.e("RSCore", "Error reading NfcV",ioe);
			throw ioe;
		}
	}
	
	public void readMifareClassic(Tag tag, byte [] key) throws IOException  {
		SECTORS = 16;
		BLOCKS = 4;
		BLOCK_SIZE = MifareClassic.BLOCK_SIZE;
		MifareClassic mifareClassicTag = MifareClassic.get(tag);
		try {
			mifareClassicTag.connect();
			Log.d("RSCore","Auth this key "+(key == null ? " null " : key.toString()));
			for (int s = 0; s < SECTORS; s++) {
				
				if (mifareClassicTag.authenticateSectorWithKeyA(s,
						key == null ? MifareClassic.KEY_DEFAULT : key )) {
					for (int b = 0; b < BLOCKS; b++) {
						int blockIndex = (s * BLOCKS) + b;
						byte [] block= mifareClassicTag
								.readBlock(blockIndex);
						for(int i=0;i<BLOCK_SIZE;i++)
							if(block == null)
								DATA += "00";
							else 
								DATA += String.format("%02X", block[i]);
					}
				}
				if(!DATA.isEmpty()) DATA += ";";
			}
			TYPE = "MifareClassic";
		} catch (IOException e) {
			throw e;
		} finally {
			if (mifareClassicTag != null) {
				try {
					mifareClassicTag.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

	}

	public String tagId() {
		String s = "";
		for (byte b : ID)
			s += String.format("%02X", b);
		return s;
	}

	@Override
	public int describeContents() {
		return 0;
	}

	@Override
	public void writeToParcel(Parcel dest, int paramInt) {
		dest.writeInt(ID.length);
		dest.writeByteArray(ID);
		dest.writeString(DATA);
		dest.writeString(TYPE);
	}
	public static final Parcelable.Creator<NfcTag> CREATOR = new Parcelable.Creator<NfcTag>() {

		@Override
		public NfcTag createFromParcel(Parcel src) {
			return new NfcTag(src);
		}

		@Override
		public NfcTag[] newArray(int size) {
			return new NfcTag[size];
		}
		
	};
}
